/* cconv - currency converter
 * by Erik Waling <erikw@acc.umu.se>
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <getopt.h>
#include <ctype.h>

#include "cconv.h"

int main(int argc, char **argv) {

    
    int optindex = 0;
    int ret;
    int i;
    
    char p_rates = 0;
    
    
    struct option long_options[] = {
        {"version"      , no_argument       , 0        , 'v'},
        {"help"         , no_argument       , 0        , 'h'},
        {"rates"        , no_argument       , 0        , 'r'},
        {"base"         , required_argument , 0        , 'b'},
        {0              , 0                 , 0        , 0  }
    };

    init_data();
    
    while ((ret = getopt_long(argc, argv, "vhrb:", long_options, &optindex)) != -1) {
        switch (ret) {
            case '?':
                printf("\nTry `cconv --help' for help.\n");
                exit(-1);
                break;
            case 'v':
                printf("cconv %s by Erik Waling <erikw@acc.umu.se> 2003-06-10\n", VERSION);
                exit(0);
                break;
            case 'h':
                print_help();
                exit(0);
                break;
            case 'r':
                p_rates = 1;
                break;
            case 'b':
                for (i=0; i<3; i++)
                    base[i] = (unsigned char) toupper(*(optarg+i));
                break;
        }
    }
    
    while (optind < argc) {
        if (isdigit(*(argv[optind]))) {
            amount = atof(argv[optind]);
            if (argv[++optind] == NULL) {
                printf("Wrong format.. Try --help\n");
                exit(-1);
            }

            for (i=0; i<3; i++) { 

                from_cur[i] = (unsigned char) toupper(*(argv[optind]+i));
            }
        }
        if ( (*(argv[optind]) == 't' || *(argv[optind]) == 'T') && (*(argv[optind]+1) == 'o' || *(argv[optind]+1) == 'O') && *(argv[optind]+2) == '\0') {
            if (argv[++optind] == NULL) {
                printf("Wrong format.. Try --help\n");
                exit(-1);
            }
            for (i=0; i<3; i++) {
                to_cur[i] = (unsigned char) toupper(*(argv[optind]+i));
            }
        }
        optind++;        
    }
    
    get_rates();
    
    if (p_rates)
        print_rates();
    
    if (amount)
        convert();

    return 0;    

}
void init_data(void) {

    memset(&curr, 0, sizeof(curr));
    memset(&datestring, 0, sizeof(datestring));
    memset(&from_cur, 0, sizeof(from_cur));
    memset(&to_cur, 0, sizeof(to_cur));
    strcpy(base, DEFAULT_BASE);
    base[3] = '\0';
    amount = 0;
    return; 
}

void get_rates(void) {

    int fd;
    struct sockaddr_in sin;
    char *databuf, *p;
    char buf[BUFSIZE];
    int len;
    
    if ((fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
        fprintf(stderr, "Failed to open socket!\nNothing to do...\n");
        exit(-1);
    }

    sin.sin_family = PF_INET;
    sin.sin_port = htons(80);
    sin.sin_addr = *(struct in_addr *) host_to_inetaddr("www.x-rates.com");
    memset(&sin.sin_zero, 0, sizeof(sin.sin_zero));
    
    if (connect(fd, (struct sockaddr *) &sin, sizeof(sin)) == -1) {
        perror("connect()");
        exit(-1);
    }

    if ((databuf = p = (char *) malloc(DATABUF)) == NULL) {
        fprintf(stderr, "malloc() error\n");
        exit(-1);
    }
    memset(databuf, 0, DATABUF);
    
    sprintf(buf, "\nGET /cgi-bin/cgicalc.cgi?value=1&base=%s HTTP/1.1\nHost: foo\nConnection: close\n\n", base);
    if (send(fd, buf, strlen(buf), 0) == -1) {
        fprintf(stderr, "Error sending request to host...\n");
        exit(-1);
    }

    while ((len = recv(fd, buf, sizeof(buf), 0)) > 0) {
        if (p-databuf+len >= DATABUF) {
            fprintf(stderr, "Too small buffer...\n");
            exit(-1);
        }
        memcpy(p, buf, len);
        p += len;
    } 
    
    if (strlen(databuf) < 5000) {
        printf("Currency code %s not found!\n",base);
    }
    
    strip_cr(databuf);
    parse_data(databuf);
    free(databuf); 
}

char *host_to_inetaddr(const char *hname) {
   
    struct hostent *hent;
    hent = gethostbyname(hname);
    
    return hent->h_addr;
}

void parse_data(char *buf) {

    int num = 0;
    char *p, *q;
    
    p = buf;
    
    while ((p = strstr(p, "<td><font face=\"Verdana\" size=-1>")) != NULL) {
        p = strchr(p, '\n');
        for (; *++p == 32; p++)
            ;
        q = strchr(p, '<');
        *q = '\0';
        strncpy(curr[num].name, p, MAX_CURRENCY_NAME_LENGTH-1);
        *q = '<';
        q = strstr(++q, "html\">");
        
        curr[num].rate = atof(q+6);
        q = strchr(q, '\n');
        q = strchr(++q, '>');
        q = strchr(++q, '>');
        strncpy(curr[num].code, ++q, 3); 
        num++;
    }
    if ((p = strstr(q, "from")) != NULL) {
        q = strchr(p, '<');
        *q = '\0';
        strncpy(datestring, p+5, sizeof(datestring)-1);
    }
    return;
}

void print_rates(void) {

    int i;
    
    printf("Value of 1 %s, List updated: %s\n\n", base, datestring);
    
    for (i = 0; curr[i].name[0] != '\0'; i++) {
        printf("%-30.30s %s %f\n", curr[i].name, curr[i].code, curr[i].rate);
    }
    putchar('\n');
    
}

void strip_cr(char *buf) {

    char *p, *q;

    if ((p = strchr(buf, '\r')) == NULL || (q = strchr(p+1, '\r')) == NULL)
        return;

    if (*(q+1) == '\n') q++;

    strcpy(p, q+1);
    strip_cr(p);
    
    return;
}

void convert(void) {

    int from, to;
    
    double k;
    

    for (from=0; (from < MAX_CURRENCIES) && ( (curr[from].name[0] != '\0') && (strncmp(curr[from].code, from_cur, 3) != 0) ); from++)
        ;
    
    for (to=0; (to < MAX_CURRENCIES) && ( (curr[to].name[0] != '\0') && (strncmp(curr[to].code, to_cur, 3) != 0) ); to++)
        ; 

    if (curr[from].rate == 0)
        if ((strncmp(from_cur, base, 3)) != 0) {
            printf("Invalid currency code: %s\n", from_cur);
            exit(-1);
        }
    
    if (curr[to].rate == 0)
        if ((strncmp(to_cur, base, 3)) != 0) {
            printf("Invalid currency code: %s\n", to_cur);
            exit(-1);    
        }
                
    k = (curr[to].rate ? curr[to].rate : 1) / (curr[from].rate ? curr[from].rate : 1); 
    
    printf("%.2f %s equals %.2f %s\n", amount, from_cur, k*amount, to_cur);

}

void print_help(void) {

    printf("cconv currency converter version %s\n", VERSION);
    printf("Usage: cconv [options] [conversion]\n\n");

    printf("Options:\n");
    printf("  -v, --version                     Prints version information\n");
    printf("  -h, --help                        Prints this text\n");
    printf("  -r, --rates                       Prints value of 1 base currency\n");
    printf("  -b, --base <CURRENCY CODE>        Set base currency, defualt USD\n\n");
    printf("Conversion:\n");
    printf("Examples: 100 usd to cad\n");
    printf("          Converts 100 US Dollars to Canadian Dollars\n\n"); 
    printf("          45.4 SEK to DKK\n");
    printf("          Converts 45.4 SEK to DKK\n\n");
            
}

    
